=== modified file 'schedule_hetero_control.py'
--- schedule_hetero_control.py	2015-08-27 21:40:15 +0000
+++ schedule_hetero_control.py	2015-09-03 19:04:07 +0000
@@ -40,9 +40,11 @@
             yield entry
 
 
-def get_candidate_version(candidate_path):
+def get_candidate_info(candidate_path):
+    """ Return candidate version and revision build number. """
     with open(os.path.join(candidate_path, 'buildvars.json')) as fp:
-        return json.load(fp)['version']
+        build_vars = json.load(fp)
+    return build_vars['version'], build_vars['revision_build']
 
 
 def calculate_jobs(root, schedule_all=False):
@@ -52,31 +54,25 @@
         parent, candidate = os.path.split(candidate_path)
         if parent != candidates_path:
             raise ValueError('Wrong path')
-        candidate_version = get_candidate_version(candidate_path)
+        candidate_version, revision_build = get_candidate_info(candidate_path)
         for release in releases:
-            if is_osx_client(candidate) is not is_osx_client(release):
-                continue
-            client_os = 'ubuntu'
-            if is_osx_client(release):
-                client_os = 'osx'
-            yield {
-                'old_version': release,  # Client
-                'candidate': candidate_version,  # Server
-                'new_to_old': 'true',
-                'candidate_path': candidate,
-                'client_os': client_os,
-            }
-            yield {
-                'old_version': release,  # Server
-                'candidate': candidate_version,  # Client
-                'new_to_old': 'false',
-                'candidate_path': candidate,
-                'client_os': client_os,
-            }
-
-
-def is_osx_client(path):
-    return path.endswith('-osx')
+            for client_os in ('ubuntu',  'osx', 'windows'):
+                yield {
+                    'old_version': release,  # Client
+                    'candidate': candidate_version,  # Server
+                    'new_to_old': 'true',
+                    'candidate_path': candidate,
+                    'client_os': client_os,
+                    'revision_build': revision_build,
+                }
+                yield {
+                    'old_version': release,  # Server
+                    'candidate': candidate_version,  # Client
+                    'new_to_old': 'false',
+                    'candidate_path': candidate,
+                    'client_os': client_os,
+                    'revision_build': revision_build,
+                }
 
 
 def build_jobs(credentials, root, jobs):

=== modified file 'test_schedule_hetero_control.py'
--- test_schedule_hetero_control.py	2015-08-27 21:40:15 +0000
+++ test_schedule_hetero_control.py	2015-09-03 19:04:07 +0000
@@ -15,7 +15,7 @@
     build_jobs,
     calculate_jobs,
     get_args,
-    get_candidate_version,
+    get_candidate_info,
     get_releases,
     )
 from utility import temp_dir
@@ -56,12 +56,13 @@
                 'http://localhost:8080', 'jrandom', 'password1')
 
 
-class TestGetCandidateVersion(TestCase):
-    def test_get_candidate_version(self):
+class TestGetCandidateInfo(TestCase):
+    def test_get_candidate_info(self):
         with temp_dir() as dir_path:
             make_build_var_file(dir_path, version='1.24.3')
-            version = get_candidate_version(dir_path)
+            version, revision = get_candidate_info(dir_path)
         self.assertEqual(version, '1.24.3')
+        self.assertEqual(revision, '2870')
 
 
 class CalculateJobs(TestCase):
@@ -73,18 +74,7 @@
             os.makedirs(candidate_path)
             make_build_var_file(candidate_path, version='1.24.3')
             jobs = list(calculate_jobs(root))
-        expected = [{'new_to_old': 'true',
-                     'old_version': '1.20.11',
-                     'candidate': '1.24.3',
-                     'candidate_path': '1.24',
-                     'client_os': 'ubuntu',
-                     },
-                    {'new_to_old': 'false',
-                     'old_version': '1.20.11',
-                     'candidate': '1.24.3',
-                     'candidate_path': '1.24',
-                     'client_os': 'ubuntu'}]
-
+        expected = self.make_jobs('1.24.3', '1.20.11', '1.24')
         self.assertItemsEqual(jobs, expected)
 
     def test_calculate_jobs_schedule_all(self):
@@ -101,66 +91,35 @@
             os.utime(buildvars_path, (time(), a_week_ago))
             jobs = list(calculate_jobs(root, schedule_all=False))
             jobs_schedule_all = list(calculate_jobs(root, schedule_all=True))
-        expected = [{'new_to_old': 'true',
-                     'client_os': 'ubuntu',
-                     'old_version': '1.20.11',
-                     'candidate': '1.23.3',
-                     'candidate_path': '1.23'},
-                    {'new_to_old': 'false',
-                     'client_os': 'ubuntu',
-                     'old_version': '1.20.11',
-                     'candidate': '1.23.3',
-                     'candidate_path': '1.23'},
-                    {'new_to_old': 'true',
-                     'client_os': 'ubuntu',
-                     'old_version': '1.20.11',
-                     'candidate': '1.24.3',
-                     'candidate_path': '1.24'},
-                    {'new_to_old': 'false',
-                     'client_os': 'ubuntu',
-                     'old_version': '1.20.11',
-                     'candidate': '1.24.3',
-                     'candidate_path': '1.24'}]
-        self.assertItemsEqual(jobs, expected[2:])
+        expected = self.make_jobs('1.24.3', '1.20.11', '1.24')
+        expected.extend(self.make_jobs('1.23.3', '1.20.11', '1.23'))
+        self.assertItemsEqual(jobs, expected[:6])
         self.assertItemsEqual(jobs_schedule_all, expected)
 
     def test_calculate_jobs_osx(self):
         with temp_dir() as root:
             release_path = os.path.join(root, 'old-juju', '1.20.11')
             os.makedirs(release_path)
-            release_path = os.path.join(root, 'old-juju', '1.20.11-osx')
-            os.makedirs(release_path)
-
-            candidate_path_1 = os.path.join(root, 'candidate', '1.24.4')
-            os.makedirs(candidate_path_1)
-            make_build_var_file(candidate_path_1, '1.24.4')
-
-            candidate_path_2 = os.path.join(root, 'candidate', '1.24.4-osx')
-            os.makedirs(candidate_path_2)
-            make_build_var_file(candidate_path_2, '1.24.4')
+            candidate_path = os.path.join(root, 'candidate', '1.24.4')
+            os.makedirs(candidate_path)
+            make_build_var_file(candidate_path, '1.24.4')
             jobs = list(calculate_jobs(root, schedule_all=False))
-        expected = [{'candidate': '1.24.4',
-                     'candidate_path': '1.24.4',
-                     'client_os': 'ubuntu',
-                     'new_to_old': 'true',
-                     'old_version': '1.20.11'},
-                    {'candidate': '1.24.4',
-                     'candidate_path': '1.24.4',
-                     'client_os': 'ubuntu',
-                     'new_to_old': 'false',
-                     'old_version': '1.20.11'},
-                    {'candidate': '1.24.4',
-                     'candidate_path': '1.24.4-osx',
-                     'client_os': 'osx',
-                     'new_to_old': 'true',
-                     'old_version': '1.20.11-osx'},
-                    {'candidate': '1.24.4',
-                     'candidate_path': '1.24.4-osx',
-                     'client_os': 'osx',
-                     'new_to_old': 'false',
-                     'old_version': '1.20.11-osx'}]
+        expected = self.make_jobs('1.24.4', '1.20.11')
         self.assertItemsEqual(jobs, expected)
 
+    def make_jobs(self, candidate, old_version, candidate_path=False):
+        jobs = []
+        for client_os in ('ubuntu', 'osx', 'windows'):
+            for new_to_old in ('false', 'true'):
+                jobs.append({
+                    'candidate': candidate,
+                    'candidate_path': candidate_path or candidate,
+                    'client_os': client_os,
+                    'new_to_old': new_to_old,
+                    'old_version': old_version,
+                    'revision_build': '2870'})
+        return jobs
+
 
 def make_build_var_file(dir_path, version):
     build_vars = {"version": version, "revision_build": "2870"}

